import logging
import sqlite3

def init_db(path):
    '''Create DB if it does not exist, and connect to it'''

    db = sqlite3.connect(path)
    c = db.cursor()
    try:
        c.execute('CREATE TABLE IF NOT EXISTS test ('
                  '  id INTEGER PRIMARY KEY, '
                  '  release CHAR[20], '
                  '  arch CHAR[20], '
                  '  package char[120])')
        c.execute('CREATE TABLE IF NOT EXISTS result ('
                  '  test_id INTEGER, '
                  '  run_id CHAR[30], '
                  '  version VARCHAR[200], '
                  '  triggers TEXT, '
                  '  duration INTEGER, '
                  '  exitcode INTEGER, '
                  '  requester TEXT, '
                  '  PRIMARY KEY(test_id, run_id), '
                  '  FOREIGN KEY(test_id) REFERENCES test(id))')
        db.commit()
        logging.debug('database %s created', path)
    except sqlite3.OperationalError as e:
        if 'already exists' not in str(e):
            raise
        logging.debug('database %s already exists', path)

    return db

def get_test_id(db_con, release, arch, src):
    if not get_test_id._cache:
        # prime the cache with all test IDs; much more efficient than doing
        # thousands of individual queries
        c = db_con.cursor()
        c.execute('SELECT * FROM test')
        while True:
            row = c.fetchone()
            if row is None:
                break
            get_test_id._cache[row[1] + '/' + row[2] + '/' + row[3]] = row[0]

    cache_idx = release + '/' + arch + '/' + src
    try:
        return get_test_id._cache[cache_idx]
    except KeyError:
        # create new ID
        c = db_con.cursor()
        c.execute('INSERT INTO test VALUES (NULL, ?, ?, ?)', (release, arch, src))
        test_id = c.lastrowid
        db_con.commit()
        get_test_id._cache[cache_idx] = test_id
        return test_id


get_test_id._cache = {}
